import {ProductSystemGenerator} from "./ProductSystemGenerator";
import {ProductGeneratorResult} from "./ProductGeneratorResult";
import {TemplateElement} from "../../templates/TemplateElement";
import {ProductTemplateSystem} from "../../templates/productGamesystem/ProductTemplateSystem";
import {ProductState} from "../states/ProductState";
import {Transition} from "../transitions/Transition";
import {ProductTransition} from "../transitions/ProductTransition";
import {State} from "../states/State";
import {SimpleTemplateTransition} from "../../templates/simpleGamesystem/SimpleTemplateTransition";
import {state, transition} from "@angular/animations";
import {ScriptAccountCondition} from "../conditions/ScriptAccountCondition";
import {SimpleTemplateState} from "../../templates/simpleGamesystem/SimpleTemplateState";
import {Character} from "../../characters/Character";
import {CharacterRelation} from "../../characters/CharacterRelation";
import {ProductGenerationData} from "./ProductGenerationData";

export class TemplateProductSystemGenerator extends ProductSystemGenerator {

  templateElement: TemplateElement


  constructor(productGamesystem: ProductTemplateSystem, templateElement: TemplateElement) {
    super(productGamesystem);
    this.templateElement = templateElement;
  }

  protected assignGeneratedStatesAndTransitions(generationResult: ProductGeneratorResult) {
    const productTemplateSystem = this.productGamesystem as ProductTemplateSystem
    productTemplateSystem.transitionMap.set(this.templateElement, generationResult.transitions)
    productTemplateSystem.stateMap.set(this.templateElement, generationResult.states)
  }

  protected getTransitionConditions(transition: Transition<any>, leftSystem: boolean) {
    const templateElement = this.determineTemplateElement(leftSystem)!;
    if(transition instanceof SimpleTemplateTransition && transition.conditionMap.has(templateElement)) {
      return transition.conditionMap.get(templateElement)!
    }
    return transition.scriptAccountConditions;
  }

  protected getTransitionActions(transition: Transition<any>, leftSystem: boolean) {
    const templateElement = this.determineTemplateElement(leftSystem)!;

    if(transition instanceof SimpleTemplateTransition && transition.actionMap.has(templateElement)) {
      return transition.actionMap.get(templateElement)!
    } else {
      return transition.scriptAccountActions;
    }
  }


  protected getStateConditions(state: State<any>, leftSystem: boolean): ScriptAccountCondition[] {
    const templateElement = this.determineTemplateElement(leftSystem)!
    if(state instanceof  SimpleTemplateState && state.conditionMap.has(templateElement)) {
      return state.conditionMap.get(templateElement)!
    } else {
      return state.conditions
    }
  }

  private determineTemplateElement(leftSystem: boolean) {
    if(this.templateElement instanceof Character) {
      return this.templateElement;
    } else if(this.templateElement instanceof CharacterRelation) {
      return leftSystem ? this.templateElement.firstCharacter : this.templateElement.secondCharacter;
    }
  }
}
