import {ModelComponentCreator} from "./ModelComponentCreator";
import {GameModel} from "../../GameModel";
import {ModelComponent} from "../../ModelComponent";
import {TemplateTypeUtilities} from "../../templates/TemplateTypeUtilities";
import {Gamesystem} from "../../gamesystems/Gamesystem";
import {SimpleGamesystem} from "../../gamesystems/SimpleGamesystem";
import {ProductGamesystem} from "../../gamesystems/ProductGamesystem";
import {SimpleTemplateGamesystem} from "../../templates/simpleGamesystem/SimpleTemplateGamesystem";
import {ProductTemplateCreator} from "../../templates/productGamesystem/ProductTemplateCreator";
import {ProductTemplateSystem} from "../../templates/productGamesystem/ProductTemplateSystem";
import {TemplateType} from "../../templates/TemplateType";

export class GamesystemCreator extends ModelComponentCreator{


  constructor(context: string, gameModel: GameModel, selectedComponent: ModelComponent | undefined) {
    super(context, gameModel, selectedComponent);
  }

  createModelComponent(): ModelComponent | undefined {
    const templateType = TemplateTypeUtilities.fromString(this.context);
    let simpleGamesystem;
    if(templateType == undefined) /**Aka normal**/{
      simpleGamesystem = new SimpleGamesystem("New Simple Gamesystem", "");
    } else {
      simpleGamesystem = new SimpleTemplateGamesystem("New Simple Gamesystem", "", templateType)
    }

    if(this.selectedComponent !== undefined && this.selectedComponent instanceof Gamesystem)/**Aka productGamesystem**/ {
      const productParentsystem: ProductGamesystem = this.constructProductFromSimple(this.selectedComponent, undefined)!
      productParentsystem.addChildGamesystem(simpleGamesystem);
    } else {
      this.gameModel!.gamesystems.push(simpleGamesystem);
    }

    return simpleGamesystem;
  }

  private constructProductFromSimple(simpleGamesystem: Gamesystem<any, any>, producttemplateType: TemplateType | undefined): ProductGamesystem | undefined {
    if(!(simpleGamesystem instanceof ProductTemplateSystem) && producttemplateType != undefined) {
      console.log("Wierd, Debug")
      return ProductTemplateCreator.convertProductToTemplate(simpleGamesystem as ProductGamesystem, this.gameModel, producttemplateType);
    }

    if(simpleGamesystem instanceof ProductGamesystem) {
      console.log("Wierder Debug")
      return simpleGamesystem;
    }

    let productGamesystem: ProductGamesystem
    if(producttemplateType != undefined) {
      productGamesystem = new ProductTemplateSystem(simpleGamesystem.componentName, simpleGamesystem.componentDescription, producttemplateType);
    } else {
      productGamesystem = new ProductGamesystem(simpleGamesystem.componentName, simpleGamesystem.componentDescription);
    }

    this.constructGamesystemHierarchy(productGamesystem, simpleGamesystem as SimpleGamesystem)
    return productGamesystem;
  }

  private constructGamesystemHierarchy(productGamesystem: ProductGamesystem, simpleGamesystem: SimpleGamesystem) {
    const simpleParentGamesystem = simpleGamesystem.parentGamesystem;

    if(simpleGamesystem.states.length > 0) {
      simpleGamesystem.componentName += "(Child)";
      if(simpleParentGamesystem == undefined) {
        this.gameModel.removeGamesystem(simpleGamesystem);
        productGamesystem.addChildGamesystem(simpleGamesystem);
        this.gameModel.addGamesystem(productGamesystem);
      } else {
        simpleParentGamesystem.removeChildGamesystem(simpleGamesystem);
        productGamesystem.addChildGamesystem(simpleGamesystem);
        simpleParentGamesystem.addChildGamesystem(productGamesystem);
      }
    } else {
      if(simpleParentGamesystem == undefined) {
        this.gameModel.removeGamesystem(simpleGamesystem);
        this.gameModel.addGamesystem(productGamesystem);
      } else {
        simpleParentGamesystem.removeChildGamesystem(simpleGamesystem);
        simpleParentGamesystem.addChildGamesystem(productGamesystem);
      }
    }
  }

}
